from rest_framework import routers

from .viewsets.author import AuthorViewSet
from .viewsets.book import BookViewSet
from .viewsets.city import CityViewSet
from .viewsets.comment import CommentViewSet
from .viewsets.description import DescriptionViewSet
from .viewsets.edition import EditionViewSet
from .viewsets.language import LanguageViewSet
from .viewsets.keywords import KeywordCategoryViewSet, KeywordViewSet
from .viewsets.media import ManuscriptViewSet, MediaViewSet
from .viewsets.passage import PassageViewSet
from .viewsets.scholium import ScholiumViewSet
from .viewsets.text import TextViewSet
from .viewsets.alignment import AlignmentViewSet
from .viewsets.passage_internal_reference import PassageInternalReferenceViewSet


class MeleagerAPIRouter(routers.DefaultRouter):
    """Custom router - adjust passage URLs to use URNs"""
    
    """
    include_format_suffixes argument has to be False
    because sometimes DRF's DefaultRouter try to use 
    the last number of the sub-fragment as an autogenerated format if there isn't a / at the end
    ex: /api/scholia/urn:cts:greekLit:tlg5011.tlg001.sag:7.748.2 doesn't work
    ex2: /api/scholia/urn:cts:greekLit:tlg5011.tlg001.sag:7.748.2/ works fine
    see https://github.com/encode/django-rest-framework/blob/master/rest_framework/routers.py#L302=
    and https://www.django-rest-framework.org/api-guide/routers/#defaultrouter
    """
    include_format_suffixes = False

    def get_lookup_regex(self, viewset, lookup_prefix=""):
        out = super().get_lookup_regex(viewset, lookup_prefix)

        if viewset is PassageViewSet:
            out = r"urn:cts:greekLit:tlg7000.tlg001.ag:(?P<book>\d+).(?P<fragment>\d+)(?P<sub_fragment>[^/]*)"
        elif viewset is ScholiumViewSet:
            out = r"urn:cts:greekLit:tlg5011.tlg001.sag:(?P<book>\d+).(?P<fragment>\d+)(?P<sub_fragment>[^/.]*).(?P<number>\d+)"
        return out


router = MeleagerAPIRouter()

router.register(r"passages", PassageViewSet, basename="passage")
router.register(r"books", BookViewSet, basename="book")
router.register(r"scholia", ScholiumViewSet, basename="scholium")
router.register(r"texts", TextViewSet, basename="text")
router.register(r"media", MediaViewSet, basename="medium")
router.register(r"manuscripts", ManuscriptViewSet, basename="manuscript")
router.register(r"keywords", KeywordViewSet, basename="keyword")
router.register(r"keyword_categories", KeywordCategoryViewSet, basename="keyword_cat")
router.register(r"languages", LanguageViewSet, basename="language")
router.register(r"descriptions", DescriptionViewSet, basename="description")
router.register(r"cities", CityViewSet, basename="city")
router.register(r"authors", AuthorViewSet, basename="author")
router.register(r"comments", CommentViewSet, basename="comment")
router.register(r"editions", EditionViewSet, basename="edition")
router.register(r"alignments", AlignmentViewSet, basename="alignment")
router.register(r"passage_internal_references", PassageInternalReferenceViewSet, basename="passage_internal_reference")

urlpatterns = router.urls
